@@ -55,6 +55,13 @@ func withPodSpecAffinityEnabled() configOption {
 	}
 }
 
+func withPodSpecHostAliasesEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.PodSpecHostAliases = config.Enabled
+		return cfg
+	}
+}
+
 func withPodSpecNodeSelectorEnabled() configOption {
 	return func(cfg *config.Config) *config.Config {
 		cfg.Features.PodSpecNodeSelector = config.Enabled
@@ -69,6 +76,13 @@ func withPodSpecTolerationsEnabled() configOption {
 	}
 }
 
+func withPodSpecRuntimeClassNameEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.PodSpecRuntimeClassName = config.Enabled
+		return cfg
+	}
+}
+
 func withPodSpecSecurityContextEnabled() configOption {
 	return func(cfg *config.Config) *config.Config {
 		cfg.Features.PodSpecSecurityContext = config.Enabled
@@ -447,7 +461,7 @@ func TestPodSpecMultiContainerValidation(t *testing.T) {
 		},
 		cfgOpts: []configOption{withPodSpecFieldRefEnabled()},
 		want: &apis.FieldError{
-			Message: "volume with name \"the-name\" not mounted",
+			Message: `volume with name "the-name" not mounted`,
 			Paths:   []string{"volumes[0].name"}},
 	}, {
 		name: "Volume mounts ok when having multiple containers",
@@ -519,7 +533,7 @@ func TestPodSpecMultiContainerValidation(t *testing.T) {
 			},
 		},
 		want: &apis.FieldError{
-			Message: "volume with name \"the-name2\" not mounted",
+			Message: `volume with name "the-name2" not mounted`,
 			Paths:   []string{"volumes[1].name"},
 		}},
 	}
@@ -543,6 +557,8 @@ func TestPodSpecMultiContainerValidation(t *testing.T) {
 }
 
 func TestPodSpecFeatureValidation(t *testing.T) {
+	runtimeClassName := "test"
+
 	featureData := []struct {
 		name        string
 		featureSpec corev1.PodSpec
@@ -570,6 +586,19 @@ func TestPodSpecFeatureValidation(t *testing.T) {
 			Paths:   []string{"affinity"},
 		},
 		cfgOpts: []configOption{withPodSpecAffinityEnabled()},
+	}, {
+		name: "HostAliases",
+		featureSpec: corev1.PodSpec{
+			HostAliases: []corev1.HostAlias{{
+				IP:        "127.0.0.1",
+				Hostnames: []string{"foo.remote", "bar.remote"},
+			}},
+		},
+		err: &apis.FieldError{
+			Message: "must not set the field(s)",
+			Paths:   []string{"hostAliases"},
+		},
+		cfgOpts: []configOption{withPodSpecHostAliasesEnabled()},
 	}, {
 		name: "NodeSelector",
 		featureSpec: corev1.PodSpec{
@@ -597,6 +626,16 @@ func TestPodSpecFeatureValidation(t *testing.T) {
 			Paths:   []string{"tolerations"},
 		},
 		cfgOpts: []configOption{withPodSpecTolerationsEnabled()},
+	}, {
+		name: "RuntimeClassName",
+		featureSpec: corev1.PodSpec{
+			RuntimeClassName: &runtimeClassName,
+		},
+		err: &apis.FieldError{
+			Message: "must not set the field(s)",
+			Paths:   []string{"runtimeClassName"},
+		},
+		cfgOpts: []configOption{withPodSpecRuntimeClassNameEnabled()},
 	}, {
 		name: "PodSpecSecurityContext",
 		featureSpec: corev1.PodSpec{
@@ -1513,7 +1552,7 @@ func TestVolumeValidation(t *testing.T) {
 		},
 		want: apis.ErrMultipleOneOf("configMap", "projected"),
 	}, {
-		name: "multiple project volume single source",
+		name: "multiple projected volumes single source",
 		v: corev1.Volume{
 			Name: "foo",
 			VolumeSource: corev1.VolumeSource{
@@ -1586,7 +1625,7 @@ func TestVolumeValidation(t *testing.T) {
 				},
 			},
 		},
-		want: apis.ErrMissingOneOf("projected[0].configMap", "projected[0].secret"),
+		want: apis.ErrMissingOneOf("projected[0].configMap", "projected[0].secret", "projected[0].serviceAccountToken"),
 	}, {
 		name: "no name",
 		v: corev1.Volume{
@@ -1661,6 +1700,39 @@ func TestVolumeValidation(t *testing.T) {
 			apis.ErrMissingField("projected[0].secret.items[0].path")).Also(
 			apis.ErrMissingField("projected[1].configMap.items[0].key")).Also(
 			apis.ErrMissingField("projected[1].configMap.items[0].path")),
+	}, {
+		name: "serviceaccounttoken",
+		v: corev1.Volume{
+			Name: "foo",
+			VolumeSource: corev1.VolumeSource{
+				Projected: &corev1.ProjectedVolumeSource{
+					Sources: []corev1.VolumeProjection{{
+						ServiceAccountToken: &corev1.ServiceAccountTokenProjection{
+							Audience:          "api",
+							ExpirationSeconds: ptr.Int64(3600),
+							Path:              "token",
+						},
+					}},
+				},
+			},
+		},
+		want: nil,
+	}, {
+		name: "projection missing serviceaccounttoken values",
+		v: corev1.Volume{
+			Name: "foo",
+			VolumeSource: corev1.VolumeSource{
+				Projected: &corev1.ProjectedVolumeSource{
+					Sources: []corev1.VolumeProjection{{
+						ServiceAccountToken: &corev1.ServiceAccountTokenProjection{
+							Audience:          "api",
+							ExpirationSeconds: ptr.Int64(3600),
+						},
+					}},
+				},
+			},
+		},
+		want: apis.ErrMissingField("projected[0].serviceAccountToken.path"),
 	}}
 
 	for _, test := range tests {
